﻿using System.IO;
using System.Linq;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;
using SQLite;

namespace MyAddress01 {
  public sealed partial class MainPage : Page {
    private const int MyFontSize = 30;
    private Thickness MyMargin = new Thickness(10);
    private string DatabaseFileName =
      Path.Combine(Windows.Storage.ApplicationData.
      Current.LocalFolder.Path, "address.db");

    public MainPage() {
      this.InitializeComponent();
    }

    protected override void OnNavigatedTo(
      NavigationEventArgs e) {

      using (var conn = new SQLiteConnection(
        DatabaseFileName)) {
        conn.CreateTable<TAddress>();
        var query = conn.Table<TAddress>();
        var result = query.ToList();

        foreach (var item in result) {
          FlipViewMain.Items.Add(MakeNewGrid(item));
        }
        UpdateTitle();
      }
    }

    private Grid MakeNewGrid(TAddress address) {
      var grid = new Grid();

      ColumnDefinition cd;
      cd = new ColumnDefinition();
      cd.Width = new Windows.UI.Xaml.GridLength(160);
      grid.ColumnDefinitions.Add(cd);
      cd = new ColumnDefinition();
      cd.Width = new Windows.UI.Xaml.GridLength(
        600, GridUnitType.Star);
      grid.ColumnDefinitions.Add(cd);

      RowDefinition rd;
      for (int i = 0; i < 5; i++) {
        rd = new RowDefinition();
        rd.Height =
          new Windows.UI.Xaml.GridLength(70);
        grid.RowDefinitions.Add(rd);
      }

      var fieldNames = new string[] {"Id", "名前", 
        "ひらがな", "郵便番号", "住所" };
      for (int i = 0; i < 5; i++) AddTextBlock(
        grid, 0, i, fieldNames[i]);
      AddTextBlock(grid, 1, 0, address.Id.ToString());
      AddTextBox(grid, 1, 1, address.Name);
      AddTextBox(grid, 1, 2, address.NameHiragana);
      AddTextBox(grid, 1, 3, address.PostalCode);
      AddTextBox(grid, 1, 4, address.Address);
      return grid;
    }

    private void AddTextBlock(Grid targetGrid,
      int column, int row, string text) {

      var tb = new TextBlock();
      tb.SetValue(Grid.ColumnProperty, column);
      tb.SetValue(Grid.RowProperty, row);
      tb.Text = text;
      tb.FontSize = MyFontSize;
      tb.Margin = MyMargin;
      targetGrid.Children.Add(tb);
    }

    private void AddTextBox(Grid targetGrid,
      int column, int row, string text) {

      var tb = new TextBox();
      tb.SetValue(Grid.ColumnProperty, column);
      tb.SetValue(Grid.RowProperty, row);
      tb.FontSize = MyFontSize;
      tb.Text = text;
      tb.Margin = MyMargin;
      targetGrid.Children.Add(tb);
    }

    private void UpdateTitle() {
      using (var conn = new SQLiteConnection(
        DatabaseFileName)) {
        conn.CreateTable<TAddress>();
        var query = conn.Table<TAddress>();
        var result = query.ToList();
        TextBlockTitle.Text = "MyAddress - " +
          result.Count.ToString() + "件";
        bool isZero = result.Count == 0;
        ButtonUpdate.IsEnabled = !isZero;
        ButtonDelete.IsEnabled = !isZero;
      }
    }

    private void ButtonNew_Click(object sender,
      RoutedEventArgs e) {

      var initial = new TAddress(0, "", "", "", "");
      FlipViewMain.Items.Add(MakeNewGrid(initial));
      FlipViewMain.SelectedIndex =
        FlipViewMain.Items.Count - 1;
      ButtonUpdate.IsEnabled = true;
    }

    private void ButtonUpdate_Click(object sender,
      RoutedEventArgs e) {

      Grid grid = (Grid)FlipViewMain.SelectedItem;
      var newRecord = new TAddress {
        Id = int.Parse(((TextBlock)grid.Children.
          ElementAt(5)).Text),
        Name = ((TextBox)grid.Children.
          ElementAt(6)).Text,
        NameHiragana = ((TextBox)grid.Children.
          ElementAt(7)).Text,
        PostalCode = ((TextBox)grid.Children.
          ElementAt(8)).Text,
        Address = ((TextBox)grid.Children.
          ElementAt(9)).Text
      };

      using (var conn = new SQLiteConnection(
        DatabaseFileName)) {
        if (newRecord.Id == 0) {
          conn.Insert(newRecord);
          ((TextBlock)grid.Children.ElementAt(5)).
            Text = newRecord.Id.ToString();
          UpdateTitle();
        } else {
          conn.Update(newRecord);
        }
      }
    }

    private void ButtonDelete_Click(object sender,
      RoutedEventArgs e) {

      Grid grid = (Grid)FlipViewMain.SelectedItem;
      using (var conn = new SQLiteConnection(
        DatabaseFileName)) {
        conn.Delete<TAddress>(int.Parse(((TextBlock)
          grid.Children.ElementAt(5)).Text));
      }

      var index = FlipViewMain.SelectedIndex;
      FlipViewMain.Items.Remove(grid);
      if (index < FlipViewMain.Items.Count)
        FlipViewMain.SelectedIndex = index;
      else
        FlipViewMain.SelectedIndex =
          FlipViewMain.Items.Count - 1;
      UpdateTitle();
    }

    private void ButtonDeleteAll_Click(
      object sender, RoutedEventArgs e) {

      using (var conn = new SQLiteConnection(
        DatabaseFileName)) {
        conn.DropTable<TAddress>();
        FlipViewMain.Items.Clear();
        conn.CreateTable<TAddress>();
        UpdateTitle();
      }
    }
  }

  public class TAddress {
    [PrimaryKey, AutoIncrement]
    public int Id { get; set; }
    [MaxLength(45)]
    public string Name { get; set; }
    [MaxLength(45)]
    public string NameHiragana { get; set; }
    [MaxLength(45)]
    public string PostalCode { get; set; }
    [MaxLength(100)]
    public string Address { get; set; }

    public TAddress() { }

    public TAddress(int id, string name,
      string nameHiragana, string postalCode,
      string address) {

      this.Id = id;
      this.Name = name;
      this.NameHiragana = nameHiragana;
      this.PostalCode = postalCode;
      this.Address = address;
    }
  }
}
