﻿using System;
using System.IO;
using System.Linq;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;
using SQLite;

namespace sqlite01 {
  public sealed partial class MainPage : Page {
    string DatabaseFileName = Path.Combine(
      Windows.Storage.ApplicationData.Current.
      LocalFolder.Path, "address.db");

    public MainPage() {
      this.InitializeComponent();
    }

    protected override void OnNavigatedTo(
      NavigationEventArgs e) {
    }

    private void ButtonCreateTable_Click(
      object sender, RoutedEventArgs e) {
      using (var conn = new SQLiteConnection(
        DatabaseFileName)) {
        conn.CreateTable<TAddress>();
      }
    }

    private void ButtonInsert_Click(object sender,
      RoutedEventArgs e) {
      using (var conn = new SQLiteConnection(
        DatabaseFileName)) {
        TAddress newRecord = new TAddress {
          Name = "田中一郎",
          NameHiragana = "たなかいちろう",
          PostalCode = "100-6101",
          Address = "東京都千代田区永田町山王" +
            "パークタワー1階0号室"
        };
        conn.Insert(newRecord);
      }
    }

    private void ButtonQuery_Click(object sender,
      RoutedEventArgs e) {
      using (var conn = new SQLiteConnection(
        DatabaseFileName)) {
        var query = conn.Table<TAddress>();
        var result = query.ToList();
        string output = "";
        foreach (var item in result) {
          output += "item.id=" + item.Id.ToString() +
            " item.Name=" + item.Name +
            " item.NameHiragana=" + item.NameHiragana +
            " item.PostalCode=" + item.PostalCode +
            " item.Address=" + item.Address +
            Environment.NewLine;
        }
        ShowMessage(output);
      }
    }

    private void ShowMessage(string s) {
      var md = new Windows.UI.Popups.MessageDialog(s);
      md.ShowAsync();
    }

    private void ButtonDropTable_Click(object sender,
      RoutedEventArgs e) {
      using (var conn = new SQLiteConnection(
        DatabaseFileName)) {
        conn.DropTable<TAddress>();
      }
    }
  }

  public class TAddress {
    [PrimaryKey, AutoIncrement]
    public int Id { get; set; }
    [MaxLength(45)]
    public string Name { get; set; }
    [MaxLength(45)]
    public string NameHiragana { get; set; }
    [MaxLength(45)]
    public string PostalCode { get; set; }
    [MaxLength(100)]
    public string Address { get; set; }
  }
}
